@@ -1,28 +1,7 @@
-# ------------------------------------------------------------------------------------------------ #
-# MIT License                                                                                      #
-#                                                                                                  #
-# Copyright (c) 2020, Microsoft Corporation                                                        #
-#                                                                                                  #
-# Permission is hereby granted, free of charge, to any person obtaining a copy of this software    #
-# and associated documentation files (the "Software"), to deal in the Software without             #
-# restriction, including without limitation the rights to use, copy, modify, merge, publish,       #
-# distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the    #
-# Software is furnished to do so, subject to the following conditions:                             #
-#                                                                                                  #
-# The above copyright notice and this permission notice shall be included in all copies or         #
-# substantial portions of the Software.                                                            #
-#                                                                                                  #
-# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING    #
-# BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND       #
-# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,     #
-# DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,   #
-# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.          #
-# ------------------------------------------------------------------------------------------------ #
-
 from gym.spaces import Box
 
 from ..utils import default_preprocessor
-from ..proba_dists import DiscretizedIntervalDist
+from ..proba_dists import DiscretizedIntervalDist, EmpiricalQuantileDist
 from ..value_transforms import ValueTransform
 from .base_stochastic_func_type1 import BaseStochasticFuncType1
 
@@ -48,15 +27,21 @@ class StochasticQ(BaseStochasticFuncType1):
 
         The gym-style environment. This is used to validate the input/output structure of ``func``.
 
-    value_range : tuple of floats
+    value_range : tuple of floats, optional
 
-        A pair of floats :code:`(min_value, max_value)`.
+        A pair of floats :code:`(min_value, max_value)`. If no :code:`value_range` is given,
+        :code:`num_bins` is the number of bins of the quantile function as in
+        `IQN <https://arxiv.org/abs/1806.06923>`_ or `QR-DQN <https://arxiv.org/abs/1710.10044>`_.
 
     num_bins : int, optional
 
-        The space of rewards is discretized in :code:`num_bins` equal sized bins. We use the default
-        setting of 51 as suggested in the `Distributional RL <https://arxiv.org/abs/1707.06887>`_
-        paper.
+        If :code:`value_range` is given: The space of rewards is discretized in :code:`num_bins`
+        equal sized bins. We use the default setting of 51 as suggested in the
+        `Distributional RL <https://arxiv.org/abs/1707.06887>`_ paper.
+
+        Else: The number of fractions of the quantile function of the rewards is defined by
+        :code:`num_bins` as in `IQN <https://arxiv.org/abs/1806.06923>`_ or
+        `QR-DQN <https://arxiv.org/abs/1710.10044>`_.
 
     observation_preprocessor : function, optional
 
@@ -89,13 +74,14 @@ class StochasticQ(BaseStochasticFuncType1):
         Seed for pseudo-random number generators.
 
     """
+
     def __init__(
-            self, func, env, value_range, num_bins=51, observation_preprocessor=None,
-            action_preprocessor=None, value_transform=None, random_seed=None):
+            self, func, env, value_range=None, num_bins=51,
+            observation_preprocessor=None, action_preprocessor=None, value_transform=None,
+            random_seed=None):
 
         self.value_transform = value_transform
-        self.value_range = self._check_value_range(value_range)
-        proba_dist = self._get_proba_dist(self.value_range, value_transform, num_bins)
+        proba_dist = self._get_proba_dist(value_transform, num_bins, value_range)
 
         # set defaults
         if observation_preprocessor is None:
@@ -122,11 +108,12 @@ class StochasticQ(BaseStochasticFuncType1):
 
     @classmethod
     def example_data(
-            cls, env, value_range, num_bins=51, observation_preprocessor=None,
-            action_preprocessor=None, value_transform=None, batch_size=1, random_seed=None):
+            cls, env, value_range, num_bins=51,
+            observation_preprocessor=None, action_preprocessor=None, value_transform=None,
+            batch_size=1, random_seed=None):
 
         value_range = cls._check_value_range(value_range)
-        proba_dist = cls._get_proba_dist(value_range, value_transform, num_bins)
+        proba_dist = cls._get_proba_dist(value_transform, num_bins, value_range)
 
         if observation_preprocessor is None:
             observation_preprocessor = default_preprocessor(env.observation_space)
@@ -252,12 +239,15 @@ class StochasticQ(BaseStochasticFuncType1):
         return super().dist_params(s, a=a)
 
     @staticmethod
-    def _get_proba_dist(value_range, value_transform, num_bins):
-        if value_transform is not None:
-            f, _ = value_transform
-            value_range = f(value_range[0]), f(value_range[1])
-        reward_space = Box(*value_range, shape=())
-        return DiscretizedIntervalDist(reward_space, num_bins)
+    def _get_proba_dist(value_transform, num_bins, value_range):
+        if value_range is not None:
+            if value_transform is not None:
+                f, _ = value_transform
+                value_range = f(value_range[0]), f(value_range[1])
+            reward_space = Box(*value_range, shape=())
+            return DiscretizedIntervalDist(reward_space, num_bins)
+        else:
+            return EmpiricalQuantileDist(num_quantiles=num_bins)
 
     @staticmethod
     def _check_value_range(value_range):
