@@ -48,8 +48,16 @@ class NormalDist(BaseProbaDist):
         Setting a finite range for :code:`clip_reals` ensures that the sigmoid doesn't fully
         saturate.
 
+    clip_logvar : pair of floats, optional
+
+        The range of values to allow for the log-variance of the distribution.
+
     """
-    def __init__(self, space, clip_box=(-256., 256.), clip_reals=(-30., 30.)):
+
+    def __init__(self, space,
+                 clip_box=(-256., 256.),
+                 clip_reals=(-30., 30.),
+                 clip_logvar=(-20., 20.)):
         if not isinstance(space, Box):
             raise TypeError(f"{self.__class__.__name__} can only be defined over Box spaces")
 
@@ -57,6 +65,7 @@ class NormalDist(BaseProbaDist):
 
         self.clip_box = clip_box
         self.clip_reals = clip_reals
+        self.clip_logvar = clip_logvar
 
         self._low = onp.maximum(onp.expand_dims(self.space.low, axis=0), self.clip_box[0])
         self._high = onp.minimum(onp.expand_dims(self.space.high, axis=0), self.clip_box[1])
@@ -81,17 +90,19 @@ class NormalDist(BaseProbaDist):
                 raise ValueError(f"expected {name}.shape: (?, {expected}), got: {x.shape}")
             if flatten:
                 x = x.reshape(x.shape[0], -1)  # batch-flatten
+            if name.startswith("logvar"):
+                x = jnp.clip(x, *self.clip_logvar)
             return x
 
         def sample(dist_params, rng):
             mu = check_shape(dist_params['mu'], name='mu', flatten=True)
             logvar = check_shape(dist_params['logvar'], name='logvar', flatten=True)
-
             X = mu + jnp.exp(logvar / 2) * jax.random.normal(rng, mu.shape)
             return X.reshape(-1, *self.space.shape)
 
         def mean(dist_params):
-            return check_shape(dist_params['mu'], name='mu', flatten=False)
+            mu = check_shape(dist_params['mu'], name='mu', flatten=False)
+            return mu
 
         def mode(dist_params):
             return mean(dist_params)
@@ -104,7 +115,8 @@ class NormalDist(BaseProbaDist):
             n = logvar.shape[-1]
             logdetvar = jnp.sum(logvar, axis=-1)  # log(det(M)) = tr(log(M))
             quadratic = jnp.einsum('ij,ij->i', jnp.square(X - mu), jnp.exp(-logvar))
-            return -0.5 * (n * log_2pi + logdetvar + quadratic)
+            logp = -0.5 * (n * log_2pi + logdetvar + quadratic)
+            return logp
 
         def entropy(dist_params):
             logvar = check_shape(dist_params['logvar'], name='logvar', flatten=True)
@@ -173,14 +185,14 @@ class NormalDist(BaseProbaDist):
         X = jnp.asarray(X, dtype=self.space.dtype)                     # ensure ndarray
         X = jnp.reshape(X, (-1, *self.space.shape))                    # ensure batch axis
         X = jnp.clip(X, self._low, self._high)                         # clip to be safe
-        X = clipped_logit((X - self._low) / (self._high - self._low))  # closed intervals -> reals
+        X = clipped_logit((X - self._low) / (self._high - self._low))  # closed intervals->reals
         return X
 
     def postprocess_variate(self, rng, X, index=0, batch_mode=False):
         X = jnp.asarray(X, dtype=self.space.dtype)                    # ensure ndarray
         X = jnp.reshape(X, (-1, *self.space.shape))                   # ensure correct shape
         X = jnp.clip(X, *self.clip_reals)                             # clip for stability
-        X = self._low + (self._high - self._low) * jax.nn.sigmoid(X)  # reals -> closed interval
+        X = self._low + (self._high - self._low) * jax.nn.sigmoid(X)  # reals->closed interval
         return X if batch_mode else onp.asanyarray(X[index])
 
     @property
