@@ -1,4 +1,5 @@
 import warnings
+from collections import Counter
 from functools import partial
 
 import chex
@@ -32,6 +33,7 @@ __all__ = (
     'single_to_batch',
     'safe_sample',
     'stack_trees',
+    'sync_shared_params',
     'tree_ravel',
     'tree_sample',
     'unvectorize',
@@ -114,9 +116,9 @@ def argmin(rng, arr, axis=-1):
 def batch_to_single(pytree, index=0):
     r"""
 
-    Extract a single instance from a :doc:`pytree <pytrees>` of array batches.
+    Extract a single instance from a :external+jax:doc:`pytree <pytrees>` of array batches.
 
-    This just does an ``leaf[index]`` on all leaf nodes of the :doc:`pytree
+    This just does an ``leaf[index]`` on all leaf nodes of the :external+jax:doc:`pytree
     <pytrees>`.
 
     Parameters
@@ -227,7 +229,7 @@ def check_preprocessors(space, *preprocessors, num_samples=20, random_seed=None)
             y0 = p0(next(rngs), x)
             for p in ps:
                 y = p(next(rngs), x)
-                if jax.tree_structure(y) != jax.tree_structure(y0):
+                if jax.tree_util.tree_structure(y) != jax.tree_util.tree_structure(y0):
                     return False
                 try:
                     jax.tree_map(test_leaves, y, y0)
@@ -514,8 +516,8 @@ def _get_leaf_diagnostics(leaf, key_prefix):
 def get_grads_diagnostics(grads, key_prefix='', keep_tree_structure=False):
     r"""
 
-    Given a :doc:`pytree <pytrees>` of grads, return a dict that contains the quantiles of the
-    magnitudes of each individual component.
+    Given a :external+jax:doc:`pytree <pytrees>` of grads, return a dict that contains the quantiles
+    of the magnitudes of each individual component.
 
     This is meant to be a high-level diagnostic. It first extracts the leaves of the pytree, then
     flattens each leaf and then it computes the element-wise magnitude. Then, it concatenates all
@@ -551,8 +553,8 @@ def get_grads_diagnostics(grads, key_prefix='', keep_tree_structure=False):
 def get_magnitude_quantiles(pytree, key_prefix=''):
     r"""
 
-    Given a :doc:`pytree <pytrees>`, return a dict that contains the quantiles of the magnitudes of
-    each individual component.
+    Given a :external+jax:doc:`pytree <pytrees>`, return a dict that contains the quantiles of the
+    magnitudes of each individual component.
 
     This is meant to be a high-level diagnostic. It first extracts the leaves of the pytree, then
     flattens each leaf and then it computes the element-wise magnitude. Then, it concatenates all
@@ -851,7 +853,7 @@ def single_to_batch(pytree):
     Take a single instance and turn it into a batch of size 1.
 
     This just does an ``np.expand_dims(leaf, axis=0)`` on all leaf nodes of the
-    :doc:`pytree <pytrees>`.
+    :external+jax:doc:`pytree <pytrees>`.
 
     Parameters
     ----------
@@ -869,6 +871,76 @@ def single_to_batch(pytree):
     return jax.tree_map(lambda arr: jnp.expand_dims(arr, axis=0), pytree)
 
 
+def sync_shared_params(*params, weights=None):
+    r"""
+    Synchronize shared params. See the :doc:`A2C stub </examples/stubs/a2c>` for example usage.
+
+    Parameters
+    ----------
+    *params : multiple hk.Params objects
+
+        The parameter dicts that contain shared parameters.
+
+    weights : list of positive floats
+
+        The relative weights to use for averaging the shared params.
+
+    Returns
+    -------
+    params : tuple of hk.Params objects
+
+        Same as input ``*params`` but with synchronized shared params.
+
+    """
+    if len(params) < 2:
+        return params
+
+    if weights is None:
+        weights = (1.0,) * len(params)
+    elif len(weights) != len(params):
+        raise ValueError(
+            f"len(weights) = {len(weights)} does not match the number of param "
+            f"dicts provided, which is {len(params)}")
+
+    # Ensure that the params are mutable.
+    params = [hk.data_structures.to_mutable_dict(p) for p in params]
+
+    # Count occurrence of top-level keys in all params.
+    key_counts = Counter(k for p in params for k in p.keys())
+
+    for k, count in key_counts.items():
+        # Skip if weights are not shared.
+        if count < 2:
+            continue
+
+        # Get shared params.
+        shared_params = [p[k] for p in params if k in p]
+        assert len(shared_params) > 1
+        chex.assert_trees_all_equal_structs(*shared_params)
+
+        # Get the relative weights.
+        relative_weights = [w for w, p in zip(weights, params) if k in p]
+        scale = sum(relative_weights)
+        assert scale > 0
+        relative_weights = [w / scale for w in relative_weights]
+
+        # Apply relative weights.
+        shared_params_weighted = [
+            jax.tree_util.tree_map(lambda x: w * x, p)
+            for w, p in zip(relative_weights, shared_params)]
+
+        # Compute the weighted average.
+        shared_params_agg = jax.tree_util.tree_map(
+            lambda *x: sum(x), *shared_params_weighted)
+
+        # Replace individual params by shared, averaged params.
+        for p in params:
+            if k in p:
+                p[k] = shared_params_agg
+
+    return tuple(hk.data_structures.to_haiku_dict(p) for p in params)
+
+
 def tree_ravel(pytree):
     r"""
 
@@ -888,7 +960,7 @@ def tree_ravel(pytree):
         A single flat array.
 
     """
-    return jnp.concatenate([jnp.ravel(leaf) for leaf in jax.tree_leaves(pytree)])
+    return jnp.concatenate([jnp.ravel(leaf) for leaf in jax.tree_util.tree_leaves(pytree)])
 
 
 def tree_sample(pytree, rng, n=1, replace=False, axis=0, p=None):
@@ -1041,7 +1113,7 @@ def unvectorize(f, in_axes=0, out_axes=0):
 
 def _check_leaf_batch_size(pytree):
     """ some boilerplate to extract the batch size with some consistency checks """
-    leaf, *leaves = jax.tree_leaves(pytree)
+    leaf, *leaves = jax.tree_util.tree_leaves(pytree)
     if not isinstance(leaf, (onp.ndarray, jnp.ndarray)) and leaf.ndim >= 1:
         raise TypeError(f"all leaves must be arrays; got type: {type(leaf)}")
     if leaf.ndim < 1:
