@@ -472,15 +472,17 @@ def generate_gif(env, filepath, policy=None, resize_to=None, duration=50, max_ep
     if isinstance(env, TrainMonitor):
         env = env.env  # unwrap to strip off TrainMonitor
 
+    assert env.render_mode == 'rgb_array', "env.render_mode must be 'rgb_array'"
+
     # collect frames
     frames = []
-    s = env.reset()
+    s, info = env.reset()
     for t in range(max_episode_steps):
         a = env.action_space.sample() if policy is None else policy(s)
-        s_next, r, done, info = env.step(a)
+        s_next, r, done, truncated, info = env.step(a)
 
         # store frame
-        frame = env.render(mode='rgb_array')
+        frame = env.render()
         frame = Image.fromarray(frame)
         frame = frame.convert('P', palette=Image.ADAPTIVE)
         if resize_to is not None:
@@ -491,13 +493,13 @@ def generate_gif(env, filepath, policy=None, resize_to=None, duration=50, max_ep
 
         frames.append(frame)
 
-        if done:
+        if done or truncated:
             break
 
         s = s_next
 
     # store last frame
-    frame = env.render(mode='rgb_array')
+    frame = env.render()
     frame = Image.fromarray(frame)
     frame = frame.convert('P', palette=Image.ADAPTIVE)
     if resize_to is not None:
